#!/usr/bin/env python

import asyncio

import pytest

from ruia import Request

def test_request_params():
    params = {
        "name": "ruia"
    }
    request = Request('http://www.httpbin.org/get', method='GET', res_type='json', params=params)
    result = asyncio.get_event_loop().run_until_complete(request.fetch())
    assert result.html['args']['name'] == "ruia"

def test_request_ua():
    headers = {
        "User-Agent": "Python3.5"
    }
    request = Request('http://www.httpbin.org/get', method='GET', res_type='json', headers=headers)
    result = asyncio.get_event_loop().run_until_complete(request.fetch())
    assert result.html['headers']['User-Agent'] == "Python3.5"




#!/usr/bin/env python

from lxml import etree


class BaseField(object):
    """
    BaseField class
    """

    def __init__(self, css_select=None, xpath_select=None, default=None):
        """
        Init BaseField class
        url: http://lxml.de/index.html
        :param css_select: css select http://lxml.de/cssselect.html
        :param xpath_select: http://www.w3school.com.cn/xpath/index.asp
        :param default: default value
        """
        self.css_select = css_select
        self.xpath_select = xpath_select
        self.default = default


class TextField(BaseField):
    """
    TextField class for a field
    """

    def __init__(self, css_select=None, xpath_select=None, default=None):
        super(TextField, self).__init__(css_select, xpath_select, default)

    def extract_value(self, html, is_source=False):
        """
        Use css_select or re_select to extract a field value
        :return:
        """
        if self.css_select:
            value = html.cssselect(self.css_select)
        elif self.xpath_select:
            value = html.xpath(self.xpath_select)
        else:
            raise ValueError('%s field: css_select or xpath_select is expected' % self.__class__.__name__)
        if is_source:
            return value
        if isinstance(value, list) and len(value) == 1:
            text = ''
            if isinstance(value[0], etree._Element):
                for node in value[0].itertext():
                    text += node.strip()
            if isinstance(value[0], str) or isinstance(value[0], etree._ElementUnicodeResult):
                text = ''.join(value)
            value = text
        if self.default is not None:
            value = value if value else self.default
        return value


class AttrField(BaseField):
    """
    AttrField class for a field
    """

    def __init__(self, attr, css_select=None, xpath_select=None, default=None):
        super(AttrField, self).__init__(css_select, xpath_select, default)
        self.attr = attr

    def extract_value(self, html, is_source=False):
        """
        Use css_select or re_select to extract a field value
        :return:
        """
        if self.css_select:
            value = html.cssselect(self.css_select)
            value = value[0].get(self.attr, value) if len(value) == 1 else value
        elif self.xpath_select:
            value = html.xpath(self.xpath_select)
        else:
            raise ValueError('%s field: css_select or xpath_select is expected' % self.__class__.__name__)
        if is_source:
            return value
        if self.default is not None:
            value = value if value else self.default
        return value



#!/usr/bin/env python

from inspect import iscoroutinefunction

from lxml import etree
from typing import Any

from ruia.field import BaseField
from ruia.request import Request


class ItemMeta(type):
    """
    Metaclass for an item
    """

    def __new__(cls, name, bases, attrs):
        __fields = dict({(field_name, attrs.pop(field_name)) for field_name, object in list(attrs.items()) if
                         isinstance(object, BaseField)})
        attrs['__fields'] = __fields
        new_class = type.__new__(cls, name, bases, attrs)
        return new_class


class Item(metaclass=ItemMeta):
    """
    Item class for each item
    """

    def __init__(self):
        self.results = {}

    @classmethod
    async def _get_html(cls, html, url, **kwargs):
        if html is None and not url:
            raise ValueError("html(url or html_etree) is expected")
        if not html:
            request = Request(url, **kwargs)
            response = await request.fetch()
            html = response.html
        return etree.HTML(html)

    @classmethod
    async def get_item(cls, *, html: str = '', url: str = '', html_etree: etree._Element = None, **kwargs) -> Any:
        if html_etree is None:
            etree_result = await cls._get_html(html, url, **kwargs)
        else:
            etree_result = html_etree
        return await cls._parse_html(etree_result)

    @classmethod
    async def get_items(cls, *, html: str = '', url: str = '', html_etree: etree._Element = None, **kwargs) -> list:
        if html_etree is None:
            etree_result = await cls._get_html(html, url, **kwargs)
        else:
            etree_result = html_etree
        items_field = getattr(cls, '__fields', {}).get('target_item', None)
        if items_field:
            items = items_field.extract_value(etree_result, is_source=True)
            if items:
                tasks = [cls._parse_html(etree_result=i) for i in items]
                all_items = []
                for task in tasks:
                    all_items.append(await task)
                return all_items
            else:
                raise ValueError("Get target_item's value error!")
        else:
            raise ValueError("target_item is expected")

    @classmethod
    async def _parse_html(cls, etree_result: etree._Element) -> object:
        if etree_result is None or not isinstance(etree_result, etree._Element):
            raise ValueError("etree._Element is expected")
        item_ins = cls()
        for field_name, field_value in getattr(item_ins, '__fields', {}).items():
            if not field_name.startswith('target_'):
                clean_method = getattr(item_ins, 'clean_%s' % field_name, None)
                value = field_value.extract_value(etree_result) if isinstance(field_value, BaseField) else field_value
                if clean_method is not None:
                    if iscoroutinefunction(clean_method):
                        value = await clean_method(value)
                    else:
                        value = clean_method(value)
                setattr(item_ins, field_name, value)
                item_ins.results[field_name] = value
        return item_ins

    def __str__(self):
        return f"<Item {self.results}>"



#!/usr/bin/env python

import os

import pytest

from ruia.wrappers import SettingsWrapper

s = SettingsWrapper()


def test_settings_load_with_file():
    s.load_with_file('./tests/settings.py')
    assert s.my_settings.get('REQUEST_CONFIG', None)['DELAY'] == 5


def test_settings_load_with_dict():
    request_config = {
        'DELAY': 10,
    }
    s.load_with_dict(request_config)
    assert s.my_settings.get('DELAY', None) == 10


def test_settings_load_with_env():
    os.environ = {
        'RUIA_DELAY':15,
    }
    s.load_from_environment()
    assert s.my_settings.get('DELAY', None) == 15



#!/usr/bin/env python

from .field import AttrField, BaseField, TextField
from .item import Item
from .middleware import Middleware
from .request import Request
from .spider import Spider

__version__ = "0.0.5"



#!/usr/bin/env python

import asyncio

import pytest

from ruia import AttrField, Item, TextField

HTML = """
<!DOCTYPE html>
<head>
    <title>豆瓣电影TOP250</title>
</head>
<body>
<ol class="grid_view">
        <li>
            <div class="item">
                <div class="pic">
                    <em class="">1</em>
                    <a href="https://movie.douban.com/subject/1292052/">
                        <img alt="肖申克的救赎" src="https://img3.doubanio.com/view/movie_poster_cover/ipst/public/p480747492.webp" class="">
                    </a>
                </div>
                <div class="info">
                    <div class="hd">
                        <a href="https://movie.douban.com/subject/1292052/" class="">
                            <span class="title">肖申克的救赎</span>
                                    <span class="title">&nbsp;/&nbsp;The Shawshank Redemption</span>
                                <span class="other">&nbsp;/&nbsp;月黑高飞(港)  /  刺激1995(台)</span>
                        </a>
                            <span class="playable">[可播放]</span>
                    </div>
                    <div class="bd">
                        <p class="">
                            导演: 弗兰克·德拉邦特 Frank Darabont&nbsp;&nbsp;&nbsp;主演: 蒂姆·罗宾斯 Tim Robbins /...<br>
                            1994&nbsp;/&nbsp;美国&nbsp;/&nbsp;犯罪 剧情
                        </p>
                        <div class="star">
                                <span class="rating5-t"></span>
                                <span class="rating_num" property="v:average">9.6</span>
                                <span property="v:best" content="10.0"></span>
                                <span>827737人评价</span>
                        </div>
                            <p class="quote">
                                <span class="inq">希望让人自由。</span>
                            </p>
                    </div>
                </div>
            </div>
        </li>
</body>
</html>
"""


class DoubanTargetItem(Item):
    target_item = TextField(css_select='div.item')
    title = TextField(css_select='span.title')
    cover = AttrField(css_select='div.pic>a>img', attr='src')
    abstract = TextField(css_select='span.inq')

    async def clean_title(self, title):
        if isinstance(title, str):
            return title
        else:
            return ''.join([i.text.strip().replace('\xa0', '') for i in title])


class DoubanItem(Item):
    title = TextField(css_select='head title')

    async def clean_title(self, title):
        return 'Title: ' + title


def test_item_from():
    item = asyncio.get_event_loop().run_until_complete(DoubanItem.get_item(html=HTML))
    assert item.title == 'Title: 豆瓣电影TOP250'


def test_target_item():
    items = asyncio.get_event_loop().run_until_complete(DoubanTargetItem.get_items(html=HTML))
    assert items[0].abstract == '希望让人自由。'



#!/usr/bin/env python

import asyncio

import aiohttp
import async_timeout

from inspect import iscoroutinefunction
from types import AsyncGeneratorType
from typing import Tuple

try:
    import uvloop

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
except ImportError:
    pass

from ruia.response import Response
from ruia.utils import get_logger


class Request(object):
    """
    Request class for each request
    """
    name = 'Request'

    # Default config
    REQUEST_CONFIG = {
        'RETRIES': 3,
        'DELAY': 0,
        'TIMEOUT': 10,
        'RETRY_FUNC': None
    }

    METHOD = ['GET', 'POST']

    def __init__(self, url: str, method: str = 'GET', *,
                 callback=None,
                 headers: dict = {},
                 metadata: dict = {},
                 request_config: dict = {},
                 request_session=None,
                 res_type: str = 'text',
                 **kwargs):
        """
        Initialization parameters
        """
        self.url = url
        self.method = method.upper()
        if self.method not in self.METHOD:
            raise ValueError('%s method is not supported' % self.method)

        self.callback = callback
        self.headers = headers
        self.metadata = metadata if metadata is not None else {}
        self.request_session = request_session
        self.request_config = request_config or self.REQUEST_CONFIG
        self.res_type = res_type
        self.kwargs = kwargs

        self.close_request_session = False
        self.logger = get_logger(name=self.name)
        self.retry_times = self.request_config.get('RETRIES', 3)
        # self.setting = SettingsWrapper()

    @property
    def current_request_func(self):
        self.logger.info(f"<{self.method}: {self.url}>")
        if self.method == 'GET':
            request_func = self.current_request_session.get(
                self.url,
                headers=self.headers,
                verify_ssl=False,
                **self.kwargs
            )
        else:
            request_func = self.current_request_session.post(
                self.url,
                headers=self.headers,
                verify_ssl=False,
                **self.kwargs
            )
        return request_func

    @property
    def current_request_session(self):
        if self.request_session is None:
            self.request_session = aiohttp.ClientSession()
            self.close_request_session = True
        return self.request_session

    async def close(self):
        if hasattr(self, "browser"):
            await self.browser.close()
        if self.close_request_session:
            await self.request_session.close()
            self.request_session = None

    async def fetch(self) -> Response:
        res_headers,res_history = {},()
        res_status = 0
        res_data, res_cookies = None, None
        if self.request_config.get('DELAY', 0) > 0:
            await asyncio.sleep(self.request_config['DELAY'])
        try:
            timeout = self.request_config.get('TIMEOUT', 10)

            async with async_timeout.timeout(timeout):
                async with self.current_request_func as resp:
                    res_status = resp.status
                    assert res_status in [200, 201]
                    if self.res_type == 'bytes':
                        res_data = await resp.read()
                    elif self.res_type == 'json':
                        res_data = await resp.json()
                    else:
                        res_data = await resp.text()
                        # content = await resp.read()
                        # charset = cchardet.detect(content)
                        # res_data = content.decode(charset['encoding'])
                    res_cookies, res_headers, res_history = resp.cookies, resp.headers, resp.history
        except Exception as e:
            self.logger.error(f"<Error: {self.url} {res_status} {str(e)}>")

        if self.retry_times > 0 and res_data is None:
            retry_times = self.request_config.get('RETRIES', 3) - self.retry_times + 1
            self.logger.info(f'<Retry url: {self.url}>, Retry times: {retry_times}')
            self.retry_times -= 1
            retry_func = self.request_config.get('RETRY_FUNC')
            if retry_func and iscoroutinefunction(retry_func):
                request_ins = await retry_func(self)
                if isinstance(request_ins, Request):
                    return await request_ins.fetch()
            return await self.fetch()

        await self.close()

        response = Response(url=self.url,
                            html=res_data,
                            metadata=self.metadata,
                            res_type=self.res_type,
                            cookies=res_cookies,
                            headers=res_headers,
                            history=res_history,
                            status=res_status)
        return response

    async def fetch_callback(self, sem) -> Tuple[AsyncGeneratorType, Response]:
        async with sem:
            res = await self.fetch()
        if self.callback is not None:
            try:
                if iscoroutinefunction(self.callback):
                    callback_res = await self.callback(res)
                    res.callback_result = callback_res
                else:
                    callback_res = self.callback(res)
            except Exception as e:
                self.logger.error(e)
                callback_res = None
        else:
            callback_res = None
        return callback_res, res

    def __str__(self):
        return "<%s %s>" % (self.method, self.url)



#!/usr/bin/env python

import asyncio

from datetime import datetime
from functools import reduce
from inspect import isawaitable
from signal import SIGINT, SIGTERM
from types import AsyncGeneratorType

from ruia.middleware import Middleware
from ruia.request import Request
from ruia.utils import get_logger

try:
    import uvloop

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
except ImportError:
    pass


class Spider:
    name = 'ruia'
    request_config = None

    failed_counts, success_counts = 0, 0
    start_urls, worker_tasks = [], []

    def __init__(self, middleware=None, loop=None):
        if not self.start_urls or not isinstance(self.start_urls, list):
            raise ValueError("Spider must have a param named start_urls, eg: start_urls = ['https://www.github.com']")
        self.logger = get_logger(name=self.name)
        self.loop = loop or asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)
        # customize middleware
        if isinstance(middleware, list):
            self.middleware = reduce(lambda x, y: x + y, middleware)
        else:
            self.middleware = middleware or Middleware()
        # async queue
        self.request_queue = asyncio.Queue()
        # semaphore
        self.sem = asyncio.Semaphore(getattr(self, 'concurrency', 3))

    async def parse(self, res):
        raise NotImplementedError

    @classmethod
    def start(cls, after_start=None, before_stop=None, middleware=None, loop=None, close_event_loop=True):
        """
        Start a spider
        :param after_start:
        :param before_stop:
        :param middleware: customize middleware or a list of middleware
        :param loop: event loop
        :return:
        """
        spider_ins = cls(middleware=middleware, loop=loop)
        spider_ins.logger.info('Spider started!')
        start_time = datetime.now()

        if after_start:
            func_after_start = after_start(spider_ins)
            if isawaitable(func_after_start):
                spider_ins.loop.run_until_complete(func_after_start)

        for _signal in (SIGINT, SIGTERM):
            try:
                spider_ins.loop.add_signal_handler(_signal, lambda: asyncio.ensure_future(spider_ins.stop(_signal)))
            except NotImplementedError:
                spider_ins.logger.warning(f'{spider_ins.name} tried to use loop.add_signal_handler '
                                          'but it is not implemented on this platform.')
        asyncio.ensure_future(spider_ins.start_master())
        try:
            spider_ins.loop.run_forever()
        finally:
            if before_stop:
                func_before_stop = before_stop(spider_ins)
                if isawaitable(func_before_stop):
                    spider_ins.loop.run_until_complete(func_before_stop)

            end_time = datetime.now()
            spider_ins.logger.info(f'Total requests: {spider_ins.failed_counts + spider_ins.success_counts}')
            if spider_ins.failed_counts:
                spider_ins.logger.info(f'Failed requests: {spider_ins.failed_counts}')
            spider_ins.logger.info(f'Time usage: {end_time - start_time}')
            spider_ins.logger.info('Spider finished!')
            spider_ins.loop.run_until_complete(spider_ins.loop.shutdown_asyncgens())
            if close_event_loop:
                spider_ins.loop.close()

    async def handle_request(self, request):
        # request middleware
        await self._run_request_middleware(request)
        # make a request
        callback_res, response = await request.fetch_callback(self.sem)
        # response middleware
        await self._run_response_middleware(request, response)
        return callback_res, response

    async def start_master(self):
        for url in self.start_urls:
            request_ins = Request(url=url,
                                  callback=self.parse,
                                  headers=getattr(self, 'headers', {}),
                                  metadata=getattr(self, 'metadata', {}),
                                  request_config=getattr(self, 'request_config'),
                                  request_session=getattr(self, 'request_session', None),
                                  res_type=getattr(self, 'res_type', 'text'),
                                  **getattr(self, 'kwargs', {}))
            # self.request_queue.put_nowait(request_ins.fetch_callback(self.sem))
            self.request_queue.put_nowait(self.handle_request(request_ins))
        workers = [asyncio.ensure_future(self.start_worker()) for i in range(2)]
        await self.request_queue.join()
        await self.stop(SIGINT)

    async def start_worker(self):
        while True:
            request_item = await self.request_queue.get()
            self.worker_tasks.append(request_item)
            if self.request_queue.empty():
                results = await asyncio.gather(*self.worker_tasks, return_exceptions=True)
                for task_result in results:
                    if not isinstance(task_result, RuntimeError):
                        callback_res, res = task_result
                        if isinstance(callback_res, AsyncGeneratorType):
                            async for request_ins in callback_res:
                                self.request_queue.put_nowait(self.handle_request(request_ins))
                        if res.html is None:
                            self.failed_counts += 1
                        else:
                            self.success_counts += 1
                self.worker_tasks = []
            self.request_queue.task_done()

    async def stop(self, _signal):
        self.logger.info(f'Stopping spider: {self.name}')
        tasks = [task for task in asyncio.Task.all_tasks() if task is not
                 asyncio.tasks.Task.current_task()]
        list(map(lambda task: task.cancel(), tasks))
        results = await asyncio.gather(*tasks, return_exceptions=True)
        self.loop.stop()

    async def _run_request_middleware(self, request):
        if self.middleware.request_middleware:
            for middleware in self.middleware.request_middleware:
                middleware_func = middleware(request)
                if isawaitable(middleware_func):
                    try:
                        result = await middleware_func
                    except Exception as e:
                        self.logger.exception(e)
                else:
                    self.logger.error('Middleware must be a coroutine function')
                    result = None

    async def _run_response_middleware(self, request, response):
        if self.middleware.response_middleware:
            for middleware in self.middleware.response_middleware:
                middleware_func = middleware(request, response)
                if isawaitable(middleware_func):
                    try:
                        result = await middleware_func
                    except Exception as e:
                        self.logger.exception(e)
                else:
                    self.logger.error('Middleware must be a coroutine function')
                    result = None



#!/usr/bin/env python

import pytest

from lxml import etree

from ruia import AttrField, TextField

HTML = """
<html>
    <head>
        <title>ruia</title>
    </head>
    <body>¬
        <p>
            <a class="test_link" href="https://github.com/howie6879/ruia">hello github.</a>
        </p>
    </body>
</html>
"""

html = etree.HTML(HTML)


def test_css_select():
    field = TextField(css_select="head title")
    value = field.extract_value(html)
    assert value == "ruia"


def test_xpath_select():
    field = TextField(xpath_select='/html/head/title')
    value = field.extract_value(html)
    assert value == "ruia"


def test_attr_field():
    attr_field = AttrField(css_select="p a.test_link", attr='href')
    value = attr_field.extract_value(html)
    assert value == "https://github.com/howie6879/ruia"



#!/usr/bin/env python

from lxml import etree


class Response(object):
    """
    Return a friendly response
    """

    def __init__(self, url: str, *,
                 metadata: dict,
                 res_type: str,
                 html: str = '',
                 cookies,
                 history,
                 headers: dict = None,
                 status: int):
        self._callback_result = None
        self._url = url
        self._metadata = metadata
        self._res_type = res_type
        self._html = html
        self._cookies = cookies
        self._history = history
        self._headers = headers
        self._status = status

    @property
    def callback_result(self):
        return self._callback_result

    @callback_result.setter
    def callback_result(self, value):
        self._callback_result = value

    @property
    def url(self):
        return self._url

    @property
    def metadata(self):
        return self._metadata

    @property
    def res_type(self):
        return self._res_type

    @property
    def html(self):
        return self._html

    @property
    def cookies(self):
        return self._cookies

    @property
    def history(self):
        return self._history

    @property
    def headers(self):
        return self._headers

    @property
    def status(self):
        return self._status

    @property
    def html_etree(self):
        html_etree = None
        if self.html:
            html_etree = etree.HTML(self.html)
        return html_etree

    def __str__(self):
        return f'<Response url[{self._res_type}]: {self._url} status:{self._status} metadata:{self._metadata}>'



#!/usr/bin/env python
import os
import re

from setuptools import find_packages, setup

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open(
        os.path.join(
            os.path.abspath(os.path.dirname(__file__)),
            'ruia/__init__.py')) as fp:
    try:
        version = re.findall(
            r"^__version__ = \"([^']+)\"\r?$", fp.read(), re.M)[0]
    except IndexError:
        raise RuntimeError('Unable to determine version.')


def read(file_name):
    with open(os.path.join(os.path.dirname(__file__), file_name)) as f:
        return f.read()


setup(
    name='ruia',
    version=version,
    author='Howie Hu',
    description="Ruia - An async web scraping micro-framework based on asyncio.",
    long_description=read('README.md'),
    author_email='xiaozizayang@gmail.com',
    install_requires=['aiohttp', 'cssselect', 'lxml'],
    url="https://github.com/howie6879/ruia/blob/master/README.md",
    packages=find_packages(),
    license='MIT',
    classifiers=[
        'Intended Audience :: Developers',
        "License :: OSI Approved :: MIT License",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    project_urls={
        'Documentation': 'https://github.com/howie6879/ruia',
        'Source': 'https://github.com/howie6879/ruia',
    },
    extras_require={
        'uvloop': ['uvloop']
    }
)



#!/usr/bin/env python

import os

from importlib import util

from ruia.utils import get_logger

logger = get_logger('settings')


class SettingsWrapper(object):
    """
    SettingsWrapper returns a spider config
    """

    def __init__(self, settings_name='settings.py'):
        self.my_settings = {}
        self.settings_name = settings_name
        self._load_settings()

    def __call__(self):
        return self.my_settings

    def settings(self):
        return self.my_settings

    def load_with_file(self, file_path):
        file_name = os.path.basename(file_path)
        if file_name[-3:] != '.py':
            logger.error("module name must be python file, such as : example.py")
        module_spec = util.spec_from_file_location(
            file_name, file_path
        )
        if module_spec is None:
            logger.error("Module path: {} not found Module:{}".format(file_name, file_path))
            return
        module = util.module_from_spec(module_spec)
        module_spec.loader.exec_module(module)
        load_settings = self._convert2dict(module)
        self.my_settings.update(load_settings)

    def load_with_dict(self, dict_params):
        self.my_settings.update(dict_params)

    def load_from_environment(self, prefix='RUIA_'):
        env_dict = {}
        for k, v in os.environ.items():
            if k.startswith(prefix):
                _, config_key = k.split(prefix, 1)
                try:
                    env_dict[config_key] = int(v)
                except ValueError:
                    try:
                        env_dict[config_key] = float(v)
                    except ValueError:
                        env_dict[config_key] = v
        self.my_settings.update(env_dict)

    def _load_settings(self):
        '''
        Load the default settings
        '''
        try:
            module = self._dynamic_import(self._closest_file(self.settings_name))
            self.my_settings = self._convert2dict(module)
        except ImportError:
            logger.warning("No default settings found")

    def _dynamic_import(self, module_path):
        basename = os.path.basename(module_path)

        if basename[-3:] == '.py':
            basename = basename[:-3]

        module_spec = util.spec_from_file_location(
            basename, module_path
        )

        if module_spec is None:
            logger.error("Module path: {} not found Module:{}".format(module_path, basename))
            return
        module = util.module_from_spec(module_spec)
        module_spec.loader.exec_module(module)
        return module

    def _convert2dict(self, module):
        res = {}
        m = dir(module)
        for key in m:
            if key.startswith('__'):
                continue
            value = getattr(module, key)
            res[key] = value
        return res

    def _closest_file(self, file_name='settings.py', path='.', prev_path=None):
        """
        return the path  of the closest settings.py file
        :param file_name: default  get file name
        :param path:
        :param prev_path:
        :return:
        """
        if path == prev_path:
            return ''

        path = os.path.abspath(path)
        settings_file = os.path.join(path, file_name)
        if os.path.exists(settings_file):
            return settings_file
        return self._closest_file(file_name=file_name, path=os.path.dirname(path), prev_path=path)
